import otelAPI from "@opentelemetry/api";
import winston from "winston";
import "@opentelemetry/winston-transport"; // install is required for winston to report to OTEL
import expressWinston from "express-winston";

const ddTracingFormat = function () {
    return winston.format(info => {
        const span = otelAPI.trace.getSpan(otelAPI.context.active());
        if (span) {
            const {spanId, traceId} = span.spanContext();
            const traceIdEnd = traceId.slice(traceId.length / 2);
            info['dd.trace_id'] = BigInt(`0x${traceIdEnd}`).toString();
            info['dd.span_id'] = BigInt(`0x${spanId}`).toString();
        }
        return info;
    })();
}

const logger = winston.createLogger({
    level: "debug",
    transports: [
        new winston.transports.Console(),
    ],
    format: winston.format.combine(ddTracingFormat(), winston.format.json())
});

const tmpLoggingMiddleware = expressWinston.logger({
    winstonInstance: logger,
    statusLevels: true
});

export const loggingMiddleware = function (...args: any[]) {
    // @ts-ignore
    return tmpLoggingMiddleware(...args)
}

// TODO don't do this in production systems without safeguards as it might be a large file.
expressWinston.responseWhitelist.push('body'); // This does not work for the proxy container for some reason.

export default logger;
